Enum pallet_collective::pallet::Call [−][src]
pub enum Call<T: Config<I>, I: 'static = ()> {
set_members {
new_members: Vec<T::AccountId>,
prime: Option<T::AccountId>,
old_count: MemberCount,
},
execute {
proposal: Box<<T as Config<I>>::Proposal>,
length_bound: u32,
},
propose {
threshold: MemberCount,
proposal: Box<<T as Config<I>>::Proposal>,
length_bound: u32,
},
vote {
proposal: T::Hash,
index: ProposalIndex,
approve: bool,
},
close {
proposal_hash: T::Hash,
index: ProposalIndex,
proposal_weight_bound: Weight,
length_bound: u32,
},
disapprove_proposal {
proposal_hash: T::Hash,
},
// some variants omitted
}Expand description
Contains one variant per dispatchable that can be called by an extrinsic.
Variants
Set the collective’s membership.
new_members: The new member list. Be nice to the chain and provide it sorted.prime: The prime member whose vote sets the default.old_count: The upper bound for the previous number of members in storage. Used for weight estimation.
Requires root origin.
NOTE: Does not enforce the expected MaxMembers limit on the amount of members, but
the weight estimations rely on it to estimate dispatchable weight.
WARNING:
The pallet-collective can also be managed by logic outside of the pallet through the
implementation of the trait ChangeMembers.
Any call to set_members must be careful that the member set doesn’t get out of sync
with other logic managing the member set.
Weight
O(MP + N)where:Mold-members-count (code- and governance-bounded)Nnew-members-count (code- and governance-bounded)Pproposals-count (code-bounded)
- DB:
- 1 storage mutation (codec
O(M)read,O(N)write) for reading and writing the members - 1 storage read (codec
O(P)) for reading the proposals Pstorage mutations (codecO(M)) for updating the votes for each proposal- 1 storage write (codec
O(1)) for deleting the oldprimeand setting the new one
- 1 storage mutation (codec
Fields of set_members
Dispatch a proposal from a member using the Member origin.
Origin must be a member of the collective.
Weight
O(M + P)whereMmembers-count (code-bounded) andPcomplexity of dispatchingproposal- DB: 1 read (codec
O(M)) + DB access ofproposal - 1 event
Add a new proposal to either be voted on or executed directly.
Requires the sender to be member.
threshold determines whether proposal is executed directly (threshold < 2)
or put up for voting.
Weight
O(B + M + P1)orO(B + M + P2)where:Bisproposalsize in bytes (length-fee-bounded)Mis members-count (code- and governance-bounded)- branching is influenced by
thresholdwhere:P1is proposal execution complexity (threshold < 2)P2is proposals-count (code-bounded) (threshold >= 2)
- DB:
- 1 storage read
is_member(codecO(M)) - 1 storage read
ProposalOf::contains_key(codecO(1)) - DB accesses influenced by
threshold:- EITHER storage accesses done by
proposal(threshold < 2) - OR proposal insertion (
threshold <= 2)- 1 storage mutation
Proposals(codecO(P2)) - 1 storage mutation
ProposalCount(codecO(1)) - 1 storage write
ProposalOf(codecO(B)) - 1 storage write
Voting(codecO(M))
- 1 storage mutation
- EITHER storage accesses done by
- 1 event
- 1 storage read
Fields of propose
Add an aye or nay vote for the sender to the given proposal.
Requires the sender to be a member.
Transaction fees will be waived if the member is voting on any particular proposal for the first time and the call is successful. Subsequent vote changes will charge a fee.
Weight
O(M)whereMis members-count (code- and governance-bounded)- DB:
- 1 storage read
Members(codecO(M)) - 1 storage mutation
Voting(codecO(M))
- 1 storage read
- 1 event
Fields of vote
Close a vote that is either approved, disapproved or whose voting period has ended.
May be called by any signed account in order to finish voting and close the proposal.
If called before the end of the voting period it will only close the vote if it is has enough votes to be approved or disapproved.
If called after the end of the voting period abstentions are counted as rejections unless there is a prime member set and the prime member cast an approval.
If the close operation completes successfully with disapproval, the transaction fee will be waived. Otherwise execution of the approved operation will be charged to the caller.
proposal_weight_bound: The maximum amount of weight consumed by executing the closed proposal.length_bound: The upper bound for the length of the proposal in storage. Checked viastorage::readso it issize_of::<u32>() == 4larger than the pure length.
Weight
O(B + M + P1 + P2)where:Bisproposalsize in bytes (length-fee-bounded)Mis members-count (code- and governance-bounded)P1is the complexity ofproposalpreimage.P2is proposal-count (code-bounded)
- DB:
- 2 storage reads (
Members: codecO(M),Prime: codecO(1)) - 3 mutations (
Voting: codecO(M),ProposalOf: codecO(B),Proposals: codecO(P2)) - any mutations done while executing
proposal(P1) - up to 3 events
Fields of close
Disapprove a proposal, close, and remove it from the system, regardless of its current state.
Must be called by the Root origin.
Parameters:
proposal_hash: The hash of the proposal that should be disapproved.
Complexity: O(P) where P is the number of max proposals DB Weight:
- Reads: Proposals
- Writes: Voting, Proposals, ProposalOf
Fields of disapprove_proposal
proposal_hash: T::HashImplementations
pub fn new_call_variant_set_members(
new_members: Vec<T::AccountId>,
prime: Option<T::AccountId>,
old_count: MemberCount
) -> Self
pub fn new_call_variant_set_members(
new_members: Vec<T::AccountId>,
prime: Option<T::AccountId>,
old_count: MemberCount
) -> Self
Create a call with the variant set_members.
Create a call with the variant execute.
pub fn new_call_variant_propose(
threshold: MemberCount,
proposal: Box<<T as Config<I>>::Proposal>,
length_bound: u32
) -> Self
pub fn new_call_variant_propose(
threshold: MemberCount,
proposal: Box<<T as Config<I>>::Proposal>,
length_bound: u32
) -> Self
Create a call with the variant propose.
Create a call with the variant vote.
pub fn new_call_variant_close(
proposal_hash: T::Hash,
index: ProposalIndex,
proposal_weight_bound: Weight,
length_bound: u32
) -> Self
pub fn new_call_variant_close(
proposal_hash: T::Hash,
index: ProposalIndex,
proposal_weight_bound: Weight,
length_bound: u32
) -> Self
Create a call with the variant close.
Create a call with the variant disapprove_proposal.